// -------------------------------------------------------------------------
// Copyright @ 1997 TCK Software, Incorporated
// All Rights Reserved
// -------------------------------------------------------------------------

#include "stdafx.h"
#include "vfDao.h"
#include "movieVF.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Static Data Members

// -------------------------------------------------------------------------
// CMovieHdr Constructor
// -------------------------------------------------------------------------
CMovieHdr::CMovieHdr(CMovieVF *pForm)
{
	Height(30);
	SetForm(pForm);

	m_lblTitle.Init(this, -1, "Title",		5,5,150,20);
	m_lblYear.Init(this, -1, "Year",		160,5,40,20);
	m_lblCategory.Init(this, -1, "Category",205,5,110,18);
	m_lblSummary.Init(this, -1, "Summary",	320,5,150,20);
	m_lblRating.Init(this, -1, "Rating",	465,7,45,20);
	m_lblDVD.Init(this, -1, "DVD",			510,7,30,18);
	m_lblQuality.Init(this, -1, "Quality",	555,7,50,18);

	// Lets use our BOLD Font for all of these headings
	CFont *pFontBold12 = VForm::GetFonts()->GetOrAdd(_T("Arial"), 120, TRUE);
	CFont *pFontBold10 = VForm::GetFonts()->GetOrAdd(_T("Arial"), 100, TRUE);

	m_lblTitle.Font(pFontBold12);
	m_lblYear.Font(pFontBold12);
	m_lblCategory.Font(pFontBold12);
	m_lblSummary.Font(pFontBold12);
	m_lblRating.Font(pFontBold10);
	m_lblDVD.Font(pFontBold10);
	m_lblQuality.Font(pFontBold10);

	AddCtl(&m_lblTitle);
	AddCtl(&m_lblYear);
	AddCtl(&m_lblCategory);
	AddCtl(&m_lblSummary);
	AddCtl(&m_lblRating);
	AddCtl(&m_lblDVD);
	AddCtl(&m_lblQuality);
}

// -------------------------------------------------------------------------
// CMovieRow Constructor
// -------------------------------------------------------------------------
CMovieRow::CMovieRow(CMovieVF *pForm)
{
	m_title.Init(this, IDVC_TITLE, "",			5,5,150,20);
	m_year.Init(this, IDVC_YEAR, "",			160,5,40,20);
	m_lstCategory.Init(this, IDVC_CATEGORY, "",	205,5,110,20);
	m_summary.Init(this, IDVC_SUMMARY, "",		320,5,150,20);
	m_cycleRating.Init(this, IDVC_RATING, pForm->GetRatingImgList(), 
												480,5,20,20);
	m_cycleRating.PictCount(6);
	m_chkDVD.Init(this, IDVC_DVD, "",			510,5,30,20);
	m_cycleQuality.Init(this, IDVC_QUALITY, pForm->GetQualityImgList(),
												555,5,75,20);
	m_cycleQuality.PictCount(6);

	AddCtl(&m_title);
	AddCtl(&m_year);
	AddCtl(&m_lstCategory);
	AddCtl(&m_summary);
	AddCtl(&m_cycleRating);
	AddCtl(&m_chkDVD);
	AddCtl(&m_cycleQuality);

	m_title.LimitText(50);
	m_year.LimitText(4);
}

// -------------------------------------------------------------------------
// CMovieRow Destructor
// -------------------------------------------------------------------------
CMovieRow::~CMovieRow()
{
}


// -------------------------------------------------------------------------
// CMovieData Helper functions
// -------------------------------------------------------------------------
void CMovieData::CopyFromRecordSet(CMovieSet &rSet)
{
	m_bookmark	= rSet.GetBookmark();
	
	m_title		= rSet.m_Title;
	m_nYear		= rSet.m_Year;
	m_category	= rSet.m_Category;
	m_summary	= rSet.m_Summary;
	m_nRating	= RatingToInt(rSet.m_Rating);
	m_bDVD		= rSet.m_DVD;
	m_nQuality	= rSet.m_Quality;
}

void CMovieData::CopyToRecordSet(CMovieSet &rSet)
{
	// Must already be on correct record
	// rSet.SetBookmark(m_bookmark);
	rSet.m_Title	= m_title;
	rSet.m_Year		= m_nYear;
	rSet.m_Category	= m_category;
	rSet.m_Summary	= m_summary;
	rSet.m_Rating	= IntToRating(m_nRating);
	rSet.m_DVD		= m_bDVD;
	rSet.m_Quality	= m_nQuality;
}

// -------------------------------------------------------------------------
// Convert Rating to/from Text
// -------------------------------------------------------------------------
int CMovieData::RatingToInt(LPCSTR szRating)
{
	if(!strcmp(szRating, "G"))		return 0;
	if(!strcmp(szRating, "PG"))		return 1;
	if(!strcmp(szRating, "PG-13"))	return 2;
	if(!strcmp(szRating, "R"))		return 3;
	if(!strcmp(szRating, "NC-17"))	return 4;
	if(!strcmp(szRating, "X"))		return 5;
	return 2;		// default to PG-13
}

CString CMovieData::IntToRating(int nRating)
{
	CString s1;
	switch(nRating)
	{
	case 0: s1 = "G";		break;
	case 1: s1 = "PG";		break;
	case 2: s1 = "PG-13";	break;
	case 3: s1 = "R";		break;
	case 4: s1 = "NC-17";	break;
	case 5: s1 = "X";		break;
	default: s1 = "PG-13";	break;
	}
	return s1;
}



// IMPLEMENT_DYNCREATE(CMovieVF, VForm)

BEGIN_MESSAGE_MAP(CMovieVF, VForm)
	//{{AFX_MSG_MAP(CMovieVF)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// -------------------------------------------------------------------------
// CMovieVF Constructor
// -------------------------------------------------------------------------
CMovieVF::CMovieVF()
{
	RowHeight(28);
	RowWidth(635);

	m_pDaoSet = new CMovieSet(theApp.GetDB());
	m_pCatSet = new CCategorySet(theApp.GetDB());

	CMovieHdr *pHdr = new CMovieHdr(this);
	SetHeader(pHdr, FALSE);

	// Lets use a 10 point Font	
	Font(GetFonts()->GetOrAdd(_T("Arial"), 100));

	if(!m_imgRating.Create(IDB_IMG_RATING, 16, 0, RGB(0,255,255)))
		TRACE("CMovieVF could not create image list\n");

	if(!m_imgQuality.Create(IDB_IMG_QUALITY, 71, 0, RGB(0,255,255)))
		TRACE("CMovieVF could not create cycle image list\n");

	RequeryData();
}

// -------------------------------------------------------------------------
// CMovieVF Destructor
// -------------------------------------------------------------------------
CMovieVF::~CMovieVF()
{
	if(m_pDaoSet)	delete m_pDaoSet;
	if(m_pCatSet)	delete m_pCatSet;
}

// -------------------------------------------------------------------------
// Function to create a CMovieRow
// -------------------------------------------------------------------------
VRow* CMovieVF::InitRow()
{
	CMovieRow *pRow = new CMovieRow(this);
	return(pRow); 
}


// -------------------------------------------------------------------------
// Gets the data from the Database
// -------------------------------------------------------------------------
BOOL CMovieVF::RequeryData()
{
	CString s1;
	CMovieData tmpData;

	m_pDaoSet->m_strSort = "Title";

	// Make sure the m_pDaoSet is not null
	ASSERT(m_pDaoSet != NULL);

	m_data.RemoveAll();

	if(!m_pDaoSet->IsOpen())
	{
		try
		{
			m_pDaoSet->Open();
		}
		catch(CDaoException* e)
		{
			CTckDb::DisplayDaoException(e);
			e->Delete();
			return FALSE;
		}
	}

	m_pDaoSet->MoveFirst();
	if(m_pDaoSet->IsOpen())
	{
		while(!m_pDaoSet->IsEOF())
		{
			if(!m_pDaoSet->IsDeleted())
			{
				tmpData.CopyFromRecordSet(*m_pDaoSet);
				m_data.Add(tmpData);
			}
			m_pDaoSet->MoveNext();
		}
	}

	ResetNumRows(m_data.GetSize());

	return TRUE;
}

// -------------------------------------------------------------------------
// Gets the data from the Database
// -------------------------------------------------------------------------
BOOL CMovieVF::GetCategories(CStringList& rList)
{
	m_pCatSet->m_strSort = "Category";

	// Make sure the m_pDaoSet is not null
	ASSERT(m_pCatSet != NULL);

	try
	{
		m_pCatSet->Open();
	}
	catch(CDaoException* e)
	{
		CTckDb::DisplayDaoException(e);
		e->Delete();
		m_pCatSet->Close();
		return FALSE;
	}

	rList.RemoveAll();
	if(m_pCatSet->IsOpen())
	{
		while(!m_pCatSet->IsEOF())
		{
			// bookmark = m_pDaoSet->GetBookmark();
			rList.AddTail(m_pCatSet->m_Category);
			m_pCatSet->MoveNext();
		}
	}

	m_pCatSet->Close();

	return TRUE;
}

// -------------------------------------------------------------------------
// Called when a row needs to be put on the screen
// -------------------------------------------------------------------------
BOOL CMovieVF::OnGetRow(VRow *pRow)
{
	CString s1;
	CMovieRow *r = (CMovieRow*) pRow;
	long idx;
	CMovieData *pData;

	idx = r->RowId();
	pData = &m_data[idx];

	r->m_title.Text(pData->m_title);
	s1.Format("%d", pData->m_nYear);
	r->m_year.Text(s1);
	r->m_lstCategory.Text(pData->m_category);
	r->m_summary.Text(pData->m_summary);
	r->m_cycleRating.Index(pData->m_nRating);
	r->m_chkDVD.Checked(pData->m_bDVD);
	r->m_cycleQuality.Index(pData->m_nQuality);

	return TRUE;
}

// -------------------------------------------------------------------------
// Called when a row needs to be saved (in the data)
// -------------------------------------------------------------------------
BOOL CMovieVF::OnSaveRow(VRow *pRow)
{
	CMovieRow *r = (CMovieRow*) pRow;
	long idx;
	CMovieData *pData;
	CString s1;

	idx = r->RowId();
	pData = &m_data[idx];

	pData->m_title = r->m_title.Text();
	s1.Format("%d", pData->m_nYear);
	pData->m_nYear = atoi(r->m_year.Text());
	pData->m_category = r->m_lstCategory.Text();
	pData->m_summary = r->m_summary.Text();
	pData->m_nRating = r->m_cycleRating.Index();
	pData->m_bDVD = r->m_chkDVD.Checked();
	pData->m_nQuality = r->m_cycleQuality.Index();

	try
	{
		m_pDaoSet->SetBookmark(pData->m_bookmark);
		if(m_pDaoSet->IsDeleted())
		{
			AfxMessageBox("Record has been deleted!");
			return FALSE;
		}
		m_pDaoSet->Edit();			// put record in edit mode
		pData->CopyToRecordSet(*m_pDaoSet);
		m_pDaoSet->Update();
	}
	catch (CDaoException* e)
	{
		AfxMessageBox(e->m_pErrorInfo->m_strDescription);
		e->Delete();
		return FALSE;
	}

	return TRUE;
}

// -------------------------------------------------------------------------
// Called when row(s) needs to be deleted from the data
// -------------------------------------------------------------------------
BOOL CMovieVF::OnDelete()
{
	CString s1, s2;
	int i;
	CMovieData  *pData;

	long nNumSelected = m_selectedList.Count();
	long nRowId = m_selectedList.First();

	s1.Format("You have selected to delete %ld records\nContinue?", nNumSelected);
	i = AfxMessageBox(s1, MB_ICONEXCLAMATION | MB_YESNO);
	if(i != IDYES)
		return TRUE;				// Dont let delete key pass through

	for(i=0; i<nNumSelected; i++)
	{
		pData = &m_data[nRowId];
		// Delete the selected rows
		try
		{
			m_pDaoSet->SetBookmark(pData->m_bookmark);
			if(!m_pDaoSet->IsDeleted())
			{
				m_pDaoSet->Delete();
			}
		}
		catch (CDaoException* e)
		{
			AfxMessageBox(e->m_pErrorInfo->m_strDescription);
			e->Delete();
			continue;
		}
		nRowId = m_selectedList.Next();
	}

	RequeryData();
	RefreshData();					// Updates Current Row
	return TRUE;
}

// -------------------------------------------------------------------------
// Called when a row needs to be added to the data
// -------------------------------------------------------------------------
BOOL CMovieVF::OnAddRow(VRow *pRow)
{
	CMovieRow *r = (CMovieRow*) pRow;
	long idx;
	CMovieData *pData, tmpData;
	CString s1;

	idx = r->RowId();			// This is the new row

	pData = &tmpData;

	pData->m_title = r->m_title.Text();
	s1.Format("%d", pData->m_nYear);
	pData->m_nYear = atoi(r->m_year.Text());
	pData->m_category = r->m_lstCategory.Text();
	pData->m_summary = r->m_summary.Text();
	pData->m_nRating = r->m_cycleRating.Index();
	pData->m_bDVD = r->m_chkDVD.Checked();
	pData->m_nQuality = r->m_cycleQuality.Index();

	try
	{
		m_pDaoSet->AddNew();			// add a new record
		pData->CopyToRecordSet(*m_pDaoSet);
		m_pDaoSet->Update();
		pData->m_bookmark = m_pDaoSet->GetLastModifiedBookmark();
		m_data.SetAtGrow(idx, tmpData);
	}
	catch (CDaoException* e)
	{
		AfxMessageBox(e->m_pErrorInfo->m_strDescription);
		e->Delete();
		return FALSE;
	}

	return TRUE;
}

// -------------------------------------------------------------------------
// Called when a row is initially being added
// -------------------------------------------------------------------------
void CMovieVF::OnStartAdd(VRow *pRow)
{
	CMovieRow *r = (CMovieRow*) pRow;

	r->m_title.Text("");
	r->m_year.Text("1990");
	r->m_lstCategory.Text("Drama");
	r->m_summary.Text("");
	r->m_cycleRating.Index(3);
	r->m_chkDVD.Checked(FALSE);
	r->m_cycleQuality.Index(3);
}


// -------------------------------------------------------------------------
// Called when a button is clicked
// -------------------------------------------------------------------------
BOOL CMovieVF::OnBeginList(VCtl* pCtl, CListBox* pLB)
{
	CStringList sList;
	CString s1;
	POSITION pos;

	switch(pCtl->Id())
	{
	case IDVC_CATEGORY:
		pLB->ResetContent();
		// Fill in with data from Database
		GetCategories(sList);
		pos = sList.GetHeadPosition();
		while(pos)
		{
			s1 = sList.GetNext(pos);
			pLB->AddString(s1);
		}
		break;
	}
	return TRUE;
}

